<?php

namespace Drupal\openwoo_publish\Form;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\openwoo_publish\OpenWooPublishPluginManager;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Class OpenWooPublishSettingsForm. Config form for the OpenWoo Publish module.
 *
 * @package Drupal\openwoo\Form
 */
class OpenWooPublishSettingsForm extends ConfigFormBase {

  /**
   * Config settings.
   *
   * @var string
   */
  const SETTINGS = 'openwoo_publish.settings';

  /**
   * The OpenWooPublish' plugin manager.
   *
   * @var \Drupal\openwoo_publish\OpenWooPublishPluginManager
   */
  protected $openWooPublishPluginManager;

  /**
   * SettingsForm constructor.
   *
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The config factory.
   * @param \Drupal\openwoo_publish\OpenWooPublishPluginManager $openwoo_publish_plugin_manager
   *   The OpenWoo plugin manager.
   */
  public function __construct(ConfigFactoryInterface $config_factory, OpenWooPublishPluginManager $openwoo_publish_plugin_manager) {
    parent::__construct($config_factory);
    $this->openWooPublishPluginManager = $openwoo_publish_plugin_manager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('config.factory'),
      $container->get('plugin.manager.openwoo_publish')
    );
  }

  /**
   * {@inheritDoc}
   */
  protected function getEditableConfigNames(): array {
    return [
      'openwoo_publish.settings',
    ];
  }

  /**
   * {@inheritDoc}
   */
  public function getFormId(): string {
    return 'openwoo_publish_settings';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state): array {
    $config = $this->config(static::SETTINGS);

    $form['client'] = [
      '#type' => 'select',
      '#title' => $this->t('Client'),
      '#empty_option' => $this->t('- Select -'),
      '#options' => $this->openWooPublishPluginManager->getPlugins(),
      '#description' => $this->t('Choose the provider where we publish content to.'),
      '#default_value' => $config->get('client'),
      '#ajax' => [
        'callback' => '::updateClientSettings',
        'wrapper' => 'client-settings-wrapper',
      ],
    ];

    $form['extended_logging'] = [
      '#type' => 'checkbox',
      '#title' => $this->t('Extended logging'),
      '#default_value' => $config->get('extended_logging') ?? 0,
    ];

    $form['client_settings'] = [
      '#type' => 'container',
      '#attributes' => ['id' => 'client-settings-wrapper'],
    ];

    // Add the current client settings to the form.
    $client = $form_state->getValue('client') ?? $config->get('client');
    if ($client) {
      $form['client_settings'] += $this->openWooPublishPluginManager->getFormElements($client, $form, $form_state);
    }

    return parent::buildForm($form, $form_state);
  }

  /**
   * AJAX callback to update the client settings form.
   *
   * @param array $form
   *   The form render array.
   *
   * @return array
   *   The form elements.
   */
  public function updateClientSettings(array $form): array {
    return $form['client_settings'];
  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {
    $this->config('openwoo_publish.settings')
      ->set('client', $form_state->getValue('client'))
      ->set('extended_logging', $form_state->getValue('extended_logging'))
      ->save();
    $this->openWooPublishPluginManager->submitFormElements($form_state->getValue('client'), $form['client_settings'], $form_state);
    parent::submitForm($form, $form_state);
  }

}
